@@ -11,9 +11,10 @@ import subprocess
 import tempfile
 import time
 import unicodedata
+import threading
 
 
-__version__ = '0.0.6'
+__version__ = '0.0.7'
 
 log = logging.getLogger(__name__)
 #log=logging
@@ -65,9 +66,10 @@ class Proc():
     '''
     config = None
     
-    def __init__(self, cmd, ubuntu_package=None, url=None):
+    def __init__(self, cmd, ubuntu_package=None, url=None, max_bytes_to_log=200):
         '''
         :param cmd: string ('ls -l') or list of strings (['ls','-l']) 
+        :param max_bytes_to_log: logging of stdout and stderr is limited by this value
         '''
         self.popen = None
         self.stdout = None
@@ -79,6 +81,8 @@ class Proc():
         self.is_started = False
         self.oserror = None
         self.cmd_param = cmd
+        self._thread = None
+        self.max_bytes_to_log = max_bytes_to_log
 
         if hasattr(cmd, '__iter__'):
             # cmd is string list
@@ -88,7 +92,7 @@ class Proc():
             # The shlex module currently does not support Unicode input!
             if  isinstance(cmd, unicode):
                 log.debug('unicode is normalized')
-                cmd = unicodedata.normalize('NFKD', cmd).encode('ascii','ignore')
+                cmd = unicodedata.normalize('NFKD', cmd).encode('ascii', 'ignore')
             self.cmd = shlex.split(cmd)
         self.cmd_as_string = ' '.join(self.cmd) # TODO: not perfect
         
@@ -191,14 +195,16 @@ class Proc():
             raise EasyProcessCheckInstalledError(self)
         return self
     
-    def call(self):
+    def call(self, timeout=None):
         '''
         Run command with arguments. Wait for command to complete.
-        Same as x.start().wait()
         
         :rtype: self
         '''
-        return self.start().wait()
+        self.start().wait(timeout=timeout)
+        if self.is_alive():
+            self.stop()
+        return self
 
     def start(self):
         '''
@@ -229,9 +235,19 @@ class Proc():
             log.debug('OSError exception:' + str(oserror))
             self.oserror = oserror
             raise EasyProcessError(self, 'start error')
-        
-        log.debug('process was started (pid=%s)' % (str(self.pid),))
         self.is_started = True
+        log.debug('process was started (pid=%s)' % (str(self.pid),))
+
+        def target():
+            log.debug('Thread started')        
+            self._wait4process()
+            log.debug('Thread finished')
+        
+        # original idea from here: 
+        # http://stackoverflow.com/questions/1191374/subprocess-with-timeout
+        self._thread = threading.Thread(target=target)
+        self._thread.start()
+        
         return self
 
 
@@ -246,18 +262,24 @@ class Proc():
         else:
             return False
         
-    def wait(self):
+    def wait(self, timeout=None):
         '''
         Wait for command to complete.
         
         :rtype: self
         '''
+
+        if self._thread:
+            self._thread.join(timeout=timeout)
+        return self
+
+    def _wait4process(self):
         def remove_ending_lf(s):
             if s.endswith('\n'):
                 return s[:-1]
             else:
                 return s
-            
+                   
         if self.popen:
             if USE_FILES:    
                 self.popen.wait()
@@ -279,9 +301,14 @@ class Proc():
             self.stderr = remove_ending_lf(self.stderr)
             
             log.debug('return code=' + str(self.return_code))
-            log.debug('stdout=' + self.stdout)
-            log.debug('stderr=' + self.stderr)
-        self.is_started = False
+            def limit_str(s):
+                if len(s)>self.max_bytes_to_log:
+                    warn = '[rest of output was removed, max_bytes_to_log={max_bytes_to_log}]'.format(max_bytes_to_log=self.max_bytes_to_log)
+                    s = s[:self.max_bytes_to_log] + warn
+                return s
+            log.debug('stdout=' + limit_str(self.stdout))
+            log.debug('stderr=' + limit_str(self.stderr))
+        #self.is_started = False
         return self
             
     def stop(self):
@@ -303,7 +330,7 @@ class Proc():
         :rtype: self
         '''
         if not self.is_started:
-            raise EasyProcessError(self, 'process was stopped twice!')
+            raise EasyProcessError(self, 'process was not started!')
         
         log.debug('stopping process (pid=%s cmd="%s")' % (str(self.pid), self.cmd))
         if self.popen:
